@@ -1,6 +1,6 @@
 #| nokogiri-layouts/keymaps.jl -- shell for binding group
 
-   $Id: keymaps.jl,v 1.6 2001/04/20 21:39:55 jsh Exp $
+   $Id: keymaps.jl,v 1.8 2003/01/12 20:30:49 jsh Exp $
 
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
 
@@ -24,7 +24,7 @@
 (define-structure sawfish.ui.layouts.keymaps ()
 
     (open rep
-	  gui.gtk
+	  gui.gtk-2.gtk
 	  rep.regexp
 	  sawfish.ui.slot
 	  sawfish.ui.wm
@@ -47,37 +47,39 @@
 	   (other-slots (filter (lambda (x) (not (keymap-slot-p x))) slots))
 	   (active (car keymap-slots)))
 
-      (gtk-box-pack-start hbox (gtk-label-new (_ "Context:")))
-      (gtk-box-pack-start hbox omenu)
-      (gtk-box-pack-start vbox hbox)
+      (when keymap-slots
+	(gtk-box-pack-start hbox (gtk-label-new (_ "Context:")))
+	(gtk-box-pack-start hbox omenu)
+	(gtk-box-pack-start vbox hbox)
 
-      (let loop ((rest keymap-slots)
-		 (last nil))
-	(when rest
-	  (let* ((slot (car rest))
-		 (button (gtk-radio-menu-item-new-with-label-from-widget
-			  last (beautify-keymap-name (slot-name slot)))))
-	    (gtk-menu-append menu button)
-	    (gtk-widget-show button)
-	    (gtk-signal-connect button "toggled"
-				(lambda (w)
-				  (when (gtk-check-menu-item-active w)
-				    (when active
-				      (gtk-container-remove
-				       km-vbox (slot-gtk-widget active)))
-				    (setq active slot)
-				    (gtk-box-pack-start
-				     km-vbox (slot-gtk-widget active) t t))))
-	    (set-slot-layout slot (slot-gtk-widget slot))
-	    (loop (cdr rest) button))))
+	(let loop ((rest keymap-slots)
+		   (last nil))
+	  (when rest
+	    (let* ((slot (car rest))
+		   (button (gtk-radio-menu-item-new-with-label-from-widget
+			    last (beautify-keymap-name (slot-name slot)))))
+	      (gtk-menu-shell-append menu button)
+	      (gtk-widget-show button)
+	      (g-signal-connect button "toggled"
+				  (lambda (w)
+				    (when (gtk-check-menu-item-active w)
+				      (when active
+					(gtk-container-remove
+					 km-vbox (slot-gtk-widget active)))
+				      (setq active slot)
+				      (gtk-box-pack-start
+				       km-vbox (slot-gtk-widget active) t t))))
+	      (set-slot-layout slot (slot-gtk-widget slot))
+	      (loop (cdr rest) button))))
 
-      (gtk-option-menu-set-menu omenu menu)
+	(gtk-option-menu-set-menu omenu menu)
 
-      (when active
-	(gtk-option-menu-set-history omenu 0)
-	(gtk-box-pack-start km-vbox (slot-gtk-widget active) t t))
+	(when active
+	  (gtk-option-menu-set-history omenu 0)
+	  (gtk-box-pack-start km-vbox (slot-gtk-widget active) t t))
+
+	(gtk-box-pack-start vbox km-vbox t t))
 
-      (gtk-box-pack-start vbox km-vbox t t)
       (gtk-box-pack-start vbox (layout-slots 'vbox other-slots))
 
       (gtk-widget-show-all vbox)
