@@ -1,5 +1,5 @@
 ;; workspace.jl -- similar to virtual desktops
-;; $Id: workspace.jl,v 1.159 2003/03/16 20:56:48 jsh Exp $
+;; $Id: workspace.jl,v 1.162 2003/04/03 02:50:12 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -234,8 +234,7 @@ window, one of `stop', `keep-going', `wrap-around'")
       (not (window-get w 'iconified))))
 
   (define (window-appears-in-workspace-p w space)
-    (and (window-viewable-p w)
-	 (or (window-get w 'sticky) (window-in-workspace-p w space))))
+    (or (window-get w 'sticky) (window-in-workspace-p w space)))
 
   ;; add window W to those in workspace SPACE
   (define (window-add-to-workspace w space)
@@ -537,7 +536,7 @@ window, one of `stop', `keep-going', `wrap-around'")
 	 (mapc (lambda (w)
 		 (when (window-appears-in-workspace-p w current-workspace)
 		   (swap-in w current-workspace))
-		 (when (and (window-in-workspace-p w current-workspace)
+		 (when (and (window-appears-in-workspace-p w current-workspace)
 			    (window-get w 'placed))
 		   (if (window-viewable-p w)
 		       (show-window w)
@@ -563,7 +562,7 @@ window, one of `stop', `keep-going', `wrap-around'")
 	(call-hook 'workspace-state-change-hook))))
 
   (define (select-workspace space #!optional dont-focus inner-thunk)
-    (select-workspace* space dont-focus inner-thunk))
+    (select-workspace* space #:dont-focus dont-focus #:inner-thunk inner-thunk))
 
   ;; return a list of all windows on workspace index SPACE
   (define (workspace-windows
@@ -590,6 +589,7 @@ window, one of `stop', `keep-going', `wrap-around'")
   ;; usually called from the add-window-hook; adds window W to the
   ;; current workspace (or wherever else it should go)
   (define (ws-add-window w)
+    (ws-window-activated w)
     (if (window-get w 'sticky)
 	(progn
 	  (set-window-workspaces w nil)
@@ -617,6 +617,11 @@ window, one of `stop', `keep-going', `wrap-around'")
     (unless (or (window-get w 'sticky) (window-workspaces w))
       (ws-add-window w)))
 
+  (define (ws-window-activated w)
+    (when (and showing-desktop
+	       (not (or (desktop-window-p w) (dock-window-p w))))
+      (hide-desktop)))
+
 
 ;;; Menu constructors
 
@@ -959,6 +964,7 @@ last instance remaining, then delete the actual window."
 
   (add-hook 'add-window-hook ws-add-window)
   (add-hook 'map-notify-hook ws-window-mapped)
+  (add-hook 'activate-window-hook ws-window-activated)
   (add-hook 'unmap-notify-hook ws-window-unmapped)
   (add-hook 'destroy-notify-hook ws-remove-window)
   (add-hook 'sm-window-save-functions ws-saved-state)
