@@ -1,5 +1,5 @@
 /* display.c -- display handling
-   $Id: display.c 4161 2005-01-04 15:34:25Z jsh $
+   $Id: display.c 4211 2008-01-19 12:03:08Z jkozicki $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -55,7 +55,8 @@ int preferred_depth;
 
 /* some atoms that may be useful.. */
 Atom xa_wm_state, xa_wm_change_state, xa_wm_protocols, xa_wm_delete_window,
-    xa_wm_colormap_windows, xa_wm_take_focus, xa_compound_text;
+  xa_wm_colormap_windows, xa_wm_take_focus, xa_compound_text,
+  xa_wm_net_name, xa_wm_net_icon_name, xa_utf8_string;
 
 DEFSYM(display_name, "display-name");
 DEFSYM(canonical_display_name, "canonical-display-name");
@@ -335,6 +336,9 @@ sys_init(char *program_name)
 	    xa_wm_colormap_windows = XInternAtom (dpy, "WM_COLORMAP_WINDOWS", False);
 	    xa_wm_take_focus = XInternAtom (dpy, "WM_TAKE_FOCUS", False);
 	    xa_compound_text = XInternAtom (dpy, "COMPOUND_TEXT", False);
+	    xa_wm_net_name = XInternAtom (dpy, "_NET_WM_NAME", False);
+	    xa_wm_net_icon_name = XInternAtom (dpy, "_NET_WM_ICON_NAME", False);
+	    xa_utf8_string = XInternAtom (dpy, "UTF8_STRING", False);
 
 	    if (!XShapeQueryExtension (dpy, &shape_event_base,
 				       &shape_error_base))
