@@ -1,5 +1,5 @@
 /* events.c -- Event handling
-   $Id: events.c 4184 2006-05-16 06:53:03Z jsh $
+   $Id: events.c 4223 2008-02-18 19:27:47Z jkozicki $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -21,6 +21,7 @@
 
 #include "sawmill.h"
 #include <limits.h>
+#include <stdint.h>
 #include <string.h>
 #include <time.h>
 #include <X11/extensions/shape.h>
@@ -29,6 +30,7 @@
 #endif
 #include <X11/Xresource.h>
 #include <X11/Xatom.h>
+#include <glib.h>
 
 /* Lookup table of event handlers */
 void (*event_handlers[LASTEvent])(XEvent *ev);
@@ -475,6 +477,115 @@ motion_notify (XEvent *ev)
        a motion event is actually evaluated) */
 }
 
+
+static bool
+update_window_name(Lisp_Window * w, XPropertyEvent xproperty) {
+    u_char *prop;
+    Atom actual;
+    int format;
+    long nitems, bytes_after;
+    char **text_list;
+    XTextProperty tprop;
+    int count;
+    repv str = Qnil;
+    int convert_status;
+
+    if (xproperty.state != PropertyNewValue
+        || XGetWindowProperty (dpy, w->id, xproperty.atom,
+                               0, 200, False, AnyPropertyType, &actual,
+                               &format, &nitems,
+                               &bytes_after, &prop) != Success
+        || actual == None)
+        return FALSE;
+
+    if (format != 8 || WINDOW_IS_GONE_P (w))
+    {
+        XFree (prop);
+        return FALSE;
+    }
+
+    tprop.value = prop;
+    tprop.encoding = actual;
+    tprop.format = format;
+    tprop.nitems = strlen (prop);
+
+#ifdef X_HAVE_UTF8_STRING
+    if (actual == xa_compound_text || actual == XA_STRING
+        || actual == xa_utf8_string) 
+    {
+        convert_status = Xutf8TextPropertyToTextList (dpy, &tprop, &text_list,
+                                                      &count);
+        if (convert_status >= Success && count > 0)
+            str = rep_string_dup (text_list[0]);
+        XFreeStringList(text_list);
+    }
+#else
+    if (actual == xa_compound_text || actual == XA_STRING)
+    {
+        convert_status = XmbTextPropertyToTextList (dpy, &tprop, &text_list,
+                                                    &count);
+        if (convert_status >= Success) 
+        {
+            if (count > 0)
+            {
+                char * utf8str = g_locale_to_utf8(text_list[0], -1,
+                                                  NULL, NULL, NULL);
+                if (utf8str)
+                {
+                    str = rep_string_dup (utf8str);
+                    g_free (utf8str);
+                }
+            }
+            XFreeStringList(text_list);
+        }
+    }
+#endif
+
+    XFree (prop);
+  
+    if (str == Qnil)
+        return FALSE;
+
+    if (xproperty.atom == xa_wm_net_name
+        && str != Qnil && Fequal (w->net_name, str) == Qnil)
+    {
+        w->net_name = str;
+        return TRUE;
+    }
+  
+    if (xproperty.atom == xa_wm_net_icon_name
+        && str != Qnil && Fequal (w->net_icon_name, str) == Qnil)
+    {
+        w->net_icon_name = str;
+        return TRUE;
+    }
+  
+    if (w->net_name == Qnil && xproperty.atom == XA_WM_NAME)
+    {
+        if (str == Qnil)
+            str = rep_null_string ();
+        if (Fequal (w->name, str) == Qnil
+            || Fequal (w->full_name, str) == Qnil)
+        {
+            w->full_name = w->name = str;
+            return TRUE;
+        }
+    }
+  
+    if (w->net_icon_name == Qnil && xproperty.atom == XA_WM_ICON_NAME)
+    {
+        if (str == Qnil)
+            str = rep_null_string ();
+        if (Fequal (w->icon_name, str) == Qnil)
+        {
+            w->icon_name = str;
+            return TRUE;
+        }
+    }    
+
+    return FALSE;
+}
+
 static void
 property_notify (XEvent *ev)
 {
@@ -488,72 +599,9 @@ property_notify (XEvent *ev)
 
 	switch (ev->xproperty.atom)
 	{
-	    u_char *prop;
-	    Atom actual;
-	    int format;
-	    long nitems, bytes_after;
 	    long supplied;
 	    bool old_urgency, new_urgency;
 
-	case XA_WM_NAME:
-	case XA_WM_ICON_NAME:
-	    if (ev->xproperty.state == PropertyNewValue
-		&& XGetWindowProperty (dpy, w->id, ev->xproperty.atom,
-				       0, 200, False, AnyPropertyType, &actual,
-				       &format, &nitems,
-				       &bytes_after, &prop) == Success
-		&& actual != None)
-	    {
-		if (format == 8 && !WINDOW_IS_GONE_P (w))
-		{
-		    repv str = Qnil;
-		    if (actual == xa_compound_text || actual == XA_STRING)
-		    {
-			char **text_list;
-			XTextProperty tprop;
-			int count;
-			tprop.value = prop;
-			tprop.encoding = actual;
-			tprop.format = format;
-			tprop.nitems = strlen (prop);
-			if (XmbTextPropertyToTextList (dpy, &tprop,
-						       &text_list, &count)
-			    >= Success)
-			{
-			    if (count > 0)
-			    	str = rep_string_dup (text_list[0]);
-			    XFreeStringList(text_list);
-			}
-		    }
-		    if (str == Qnil)
-			str = rep_null_string ();
-
-		    if (ev->xproperty.atom == XA_WM_NAME)
-		    {
-			if (Fequal (w->name, str) == Qnil
-			    || Fequal (w->full_name, str) == Qnil)
-			{
-			    w->full_name = w->name = str;
-			    need_refresh = TRUE;
-			}
-			else
-			    changed = FALSE;
-		    }
-		    else
-		    {
-			if (Fequal (w->icon_name, str) == Qnil)
-			{
-			    w->icon_name = str;
-			    need_refresh = TRUE;
-			}
-			else
-			    changed = FALSE;
-		    }
-		}
-		XFree (prop);
-	    }
-	    break;
-
 	case XA_WM_HINTS:
 	    old_urgency = w->wmhints != 0 && w->wmhints->flags & XUrgencyHint;
 	    if (w->wmhints != 0)
@@ -571,7 +619,15 @@ property_notify (XEvent *ev)
 	    break;
 
 	default:
-	    if (ev->xproperty.atom == xa_wm_colormap_windows)
+            if (ev->xproperty.atom == XA_WM_NAME ||
+                ev->xproperty.atom == XA_WM_ICON_NAME ||
+                ev->xproperty.atom == xa_wm_net_name ||
+                ev->xproperty.atom == xa_wm_net_icon_name ) 
+            {
+                need_refresh = changed = 
+                    update_window_name(w, ev->xproperty);
+            }
+            else if (ev->xproperty.atom == xa_wm_colormap_windows)
 	    {
 		if (w->n_cmap_windows > 0)
 		    XFree (w->cmap_windows);
@@ -641,7 +697,10 @@ client_message (XEvent *ev)
     case 32:
 	data = Fmake_vector (rep_MAKE_INT(5), Qnil);
 	for (i = 0; i < 5; i++)
-	    rep_VECTI(data,i) = rep_make_long_uint (ev->xclient.data.l[i]);
+	{
+	     unsigned long l = (uint32_t) ev->xclient.data.l[i];
+	     rep_VECTI(data,i) = rep_make_long_uint (l);
+	}
 	break;
 
     default:
@@ -689,10 +748,13 @@ map_request (XEvent *ev)
     Lisp_Window *w = find_window_by_id (id);
     if (w == 0)
     {
+        /* Also adds the frame. */
 	w = add_window (id);
 	if (w == 0)
+	{
+	    fprintf (stderr, "warning: failed to allocate a window\n");
 	    return;
-
+	}
 	if (w->wmhints && w->wmhints->flags & StateHint
 	    && w->wmhints->initial_state == IconicState)
 	{
@@ -736,12 +798,20 @@ reparent_notify (XEvent *ev)
 	if (ev->xreparent.parent != root_window
 	    && ev->xreparent.parent != w->frame)
 	{
-	    /* Not us doing the reparenting.. */
+	    /* The window is no longer on our turf and we must not
+	       reparent it to the root. -- thk */
+	    w->reparented = FALSE;
+	    XRemoveFromSaveSet (dpy, w->id);
+
+            Fcall_window_hook (Qreparent_notify_hook,
+                               rep_VAL(w), Qnil, Qnil);
+
+	    /* Not us doing the reparenting. */
 	    remove_window (w, FALSE, FALSE);
-	    XReparentWindow (dpy, ev->xreparent.window, ev->xreparent.parent,
-			     ev->xreparent.x, ev->xreparent.y);
 	}
-	Fcall_window_hook (Qreparent_notify_hook, rep_VAL(w), Qnil, Qnil);
+        else
+            Fcall_window_hook (Qreparent_notify_hook,
+                               rep_VAL(w), Qnil, Qnil);
     }
 }
 
@@ -757,6 +827,10 @@ map_notify (XEvent *ev)
 	{
 	    /* arrgh, the window changed its override redirect status.. */
 	    remove_window (w, FALSE, FALSE);
+#if 0
+	    fprintf(stderr, "warning: I've had it with window %#lx\n",
+		    (long)(w->id));
+#endif
 	}
 	else
 	{
@@ -765,11 +839,11 @@ map_notify (XEvent *ev)
 	    w->attr.height = wa.height;
 
 	    w->mapped = TRUE;
+	    /* This should not happen.  The window should have been
+	       framed at the map request. -- thk */
 	    if (w->frame == 0)
-		create_window_frame (w);
-	    install_window_frame (w);
-	    if (w->visible)
-		XMapWindow (dpy, w->frame);
+		fprintf (stderr, "warning: window %#1x has no frame\n",
+			 (long)(w->id));
 	    Fcall_window_hook (Qmap_notify_hook, rep_VAL(w), Qnil, Qnil);
 	}
     }
@@ -782,6 +856,9 @@ unmap_notify (XEvent *ev)
     if (w != 0 && ev->xunmap.window == w->id
 	&& (ev->xunmap.event == w->id || ev->xunmap.send_event))
     {
+	int being_reparented = FALSE;
+	XEvent reparent_ev;
+
 	w->mapped = FALSE;
 	if (w->reparented)
 	{
@@ -790,19 +867,45 @@ unmap_notify (XEvent *ev)
 		XUnmapWindow (dpy, w->frame);
 		reset_frame_parts (w);
 	    }
-	    /* Removing the frame reparents the client window back to
-	       the root. This means that we receive the next MapRequest
-	       for the window. */
-	    remove_window_frame (w);
-	    destroy_window_frame (w, FALSE);
+	    /* Careful now.  It is possible that the unmapping was
+	       caused by someone else reparenting the window.
+	       Removing the frame involves reparenting the window to
+	       the root.  Bad things may happen if we do that while
+	       a different reparenting is in progress. -- thk */
+	    being_reparented = XCheckTypedWindowEvent (dpy, w->id,
+						       ReparentNotify,
+						       &reparent_ev);
+	    if (!being_reparented)
+	    {
+	        /* Removing the frame reparents the client window back to
+		   the root. This means that we receive the next MapRequest
+		   for the window. */
+		remove_window_frame (w);
+		destroy_window_frame (w, FALSE);
+	    }
+
+	    /* Handle a possible race condition: if the client
+	       withdrew the window while we were in the process of
+	       mapping it, the window may be mapped now.  -- thk */
+	    if (ev->xunmap.send_event && !w->client_unmapped)
+	    {
+		before_local_map (w);
+		XUnmapWindow (dpy, w->id);
+		after_local_map (w);
+	    }
 	}
 	Fcall_window_hook (Qunmap_notify_hook, rep_VAL(w), Qnil, Qnil);
 
-	XDeleteProperty (dpy, w->id, xa_wm_state);
-
 	/* Changed the window-handling model, don't let windows exist
 	   while they're withdrawn */
-	remove_window (w, FALSE, FALSE);
+	if (being_reparented)
+	    reparent_notify(&reparent_ev);
+	else
+	    remove_window (w, FALSE, FALSE);
+
+	/* This lets the client know that we are done, in case it wants
+	   to reuse the window. */
+	XDeleteProperty (dpy, ev->xunmap.window, xa_wm_state);
     }
 }
 
@@ -953,13 +1056,16 @@ static void
 focus_out (XEvent *ev)
 {
     Lisp_Window *w = find_window_by_id (ev->xfocus.window);
-    if (ev->xfocus.detail == NotifyPointer)
+    if (ev->xfocus.detail == NotifyPointer ||
+	ev->xfocus.mode == NotifyGrab || ev->xfocus.mode == NotifyUngrab)
 	return;
     if (w != 0 && ev->xfocus.detail != NotifyInferior)
     {
 	if (focus_window == w)
 	{
-	    focus_window = 0;
+	    if (ev->xfocus.mode == NotifyNormal || \
+		ev->xfocus.mode == NotifyWhileGrabbed)
+		focus_window = 0;
 	    report_focus_change (w);
 	}
 
@@ -1155,7 +1261,11 @@ shape_notify (XEvent *ev)
     XShapeEvent *sev = (XShapeEvent *)ev;
     Lisp_Window *w = find_window_by_id (sev->window);
     if (w != 0 && sev->window == w->id
-	&& (sev->kind == ShapeBounding || sev->kind == ShapeInput))
+	&& (sev->kind == ShapeBounding 
+#ifdef ShapeInput
+	    || sev->kind == ShapeInput
+#endif
+	    ))
     {
 	if (sev->kind == ShapeBounding)
 	    w->shaped = sev->shaped ? 1 : 0;
