@@ -1,5 +1,5 @@
 /* frames.c -- window frame manipulation
-   $Id: frames.c,v 1.106 2002/11/03 21:00:36 jsh Exp $
+   $Id: frames.c 4184 2006-05-16 06:53:03Z jsh $
 
    Copyright (C) 1999, 2000 John Harper <john@dcs.warwick.ac.uk>
 
@@ -38,6 +38,7 @@
 #endif
    
 #include "sawmill.h"
+#include <string.h>
 #include <X11/Xutil.h>
 #include <X11/Xresource.h>
 #include <X11/extensions/shape.h>
@@ -377,6 +378,14 @@ set_frame_shapes (Lisp_Window *w, bool atomic)
 	    rects[nrects].height = w->attr.height;
 	    nrects++;
 	}
+#ifdef ShapeInput
+       	XShapeCombineShape (dpy, shape_win, ShapeInput,
+			    -w->frame_x, -w->frame_y, w->id,
+			    ShapeBounding, ShapeSubtract);
+       	XShapeCombineShape (dpy, shape_win, ShapeInput,
+			    -w->frame_x, -w->frame_y, w->id,
+			    ShapeInput, ShapeUnion);
+#endif
     }
 
     for (fp = w->frame_parts; fp != 0 && !WINDOW_IS_GONE_P (w); fp = fp->next)
@@ -465,6 +474,10 @@ set_frame_shapes (Lisp_Window *w, bool atomic)
     {
 	XShapeCombineShape (dpy, w->frame, ShapeBounding,
 			    0, 0, shape_win, ShapeBounding, ShapeSet);
+#ifdef ShapeInput
+	XShapeCombineShape (dpy, w->frame, ShapeInput,
+			    0, 0, shape_win, ShapeInput, ShapeSet);
+#endif
 	XDestroyWindow (dpy, shape_win);
     }
 
@@ -869,8 +882,12 @@ refresh_frame_parts (Lisp_Window *w)
 struct frame_part *
 find_frame_part_by_window (Window id)
 {
-    struct frame_part *fp;
-    return XFindContext (dpy, id, window_fp_context, (XPointer *)&fp) ? 0 : fp;
+    union {
+	struct frame_part *f;
+	XPointer p;
+    } fp;
+
+    return XFindContext (dpy, id, window_fp_context, &fp.p) ? 0 : fp.f;
 }
 
 /* Destroy the window frame of W, assuming it's a frame-part derived frame */
@@ -1549,17 +1566,30 @@ list_frame_generator (Lisp_Window *w)
 
     /* create the child-of-root frame window, or if it already exists,
        configure it to the correct size.. */
+
     if (w->frame == 0)
     {
-	/* create the frame */
+	int depth = image_depth;
+	Visual *visual = image_visual;
+	Colormap colormap = image_cmap;
+
+	/* If window is using an ARGB visual, the frame also should. */
+	if (w->attr.depth == 32)
+	{
+	    depth = 32;
+	    visual = w->attr.visual;
+	    colormap = w->attr.colormap;
+	}
+
 	wa.override_redirect = True;
-	wa.colormap = image_cmap;
+	wa.colormap = colormap;
 	wa.border_pixel = BlackPixel (dpy, screen_num);
-	wamask = CWOverrideRedirect | CWColormap | CWBorderPixel;
+	wa.save_under = w->attr.save_under;
+	wamask = CWOverrideRedirect | CWColormap | CWBorderPixel | CWSaveUnder;
+
 	w->frame = XCreateWindow (dpy, root_window, w->attr.x, w->attr.y,
-				  w->frame_width, w->frame_height,
-				  0, image_depth, InputOutput,
-				  image_visual, wamask, &wa);
+				  w->frame_width, w->frame_height, 0,
+				  depth, InputOutput, visual, wamask, &wa);
     }
     else
     {
