@@ -1,5 +1,5 @@
 /* gtk-style.c -- print the default GTK rc style (if possible) as s-exprs
-   $Id: gtk-style.c,v 1.3 2000/12/31 16:39:58 jsh Exp $
+   $Id: gtk-style.c 4057 2003-01-12 20:35:23Z jsh $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -95,10 +95,6 @@ print_strings (char *name, char **x, int n)
 void
 print_rc_style (GtkRcStyle *rc, GtkStyle *style)
 {
-    if (rc->font_name != 0)
-	printf ("(font . \"%s\")\n", rc->font_name);
-    if (rc->fontset_name != 0)
-	printf ("(fontset . \"%s\")\n", rc->fontset_name);
     if (print_rc_colors ("fg", rc->fg, 5, rc->color_flags, GTK_RC_FG))
     	print_colors ("fg", style->fg, 5);
     if (print_rc_colors ("bg", rc->bg, 5, rc->color_flags, GTK_RC_BG))
