@@ -1,5 +1,5 @@
 /* session.c -- session manager skeleton
-   $Id: session.c,v 1.12 2000/09/03 12:43:08 john Exp $
+   $Id: session.c 3493 2001-02-12 20:09:48Z jsh $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -140,11 +140,14 @@ Deletes the session manager property called PROPERTY-NAME (a string).
 
 #ifdef HAVE_X11_SM_SMLIB_H
 
+static bool outstanding_save_done;
+
 static void
 save_yourself_2 (SmcConn conn, SmPointer data)
 {
     repv ret = rep_call_lisp0 (global_symbol_value (Qsm_save_yourself));
     SmcSaveYourselfDone (conn, (ret && ret != Qnil) ? True : False);
+    outstanding_save_done = FALSE;
 }
 
 static void
@@ -152,7 +155,10 @@ save_yourself (SmcConn conn, SmPointer data, int save_type, Bool shutdown,
 	       int interact_style, Bool fast)
 {
     if (save_type == SmSaveLocal || save_type == SmSaveBoth)
+    {
+	outstanding_save_done = TRUE;
 	SmcRequestSaveYourselfPhase2 (conn, &save_yourself_2, 0);
+    }
     else
 	SmcSaveYourselfDone (conn, True);
 }
@@ -178,6 +184,11 @@ save_complete (SmcConn conn, SmPointer data)
 static void
 shutdown_cancelled (SmcConn conn, SmPointer data)
 {
+    if (outstanding_save_done)
+    {
+	SmcSaveYourselfDone (conn, True);
+	outstanding_save_done = FALSE;
+    }
 }
 
 #endif /* HAVE_X11_SM_SMLIB_H */
