@@ -1,5 +1,5 @@
 /* x.c -- raw X manipulation
-   $Id: x.c,v 1.23 2002/11/03 02:22:43 jsh Exp $
+   $Id: x.c 4161 2005-01-04 15:34:25Z jsh $
 
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
 
@@ -41,6 +41,7 @@
 #endif
    
 #include "sawmill.h"
+#include <string.h>
 #include <X11/Xresource.h>
 
 #ifdef HAVE_X11_EXTENSIONS_XDBE_H
@@ -157,9 +158,13 @@ DEFSYM (set, "set");
 static inline repv
 x_window_from_id (Window id)
 {
-    repv win;
+    union {
+	repv r;
+	XPointer p;
+    } win;
+
     return XFindContext (dpy, id, x_drawable_context,
-			 (XPointer *) &win) ? Qnil : win;
+			 &win.p) ? Qnil : win.r;
 }
 
 #ifdef HAVE_X11_EXTENSIONS_XDBE_H
